@@ -20,7 +20,8 @@ pixelplot.mesh <- function(...) {
 #'
 #' @examples
 #' \dontrun{
-#' if (require("ggplot2", quietly = TRUE)) {
+#' if (require("ggplot2", quietly = TRUE) &&
+#'   require(ggpolypath, quietly = TRUE)) {
 #'   # Load the Gorilla data
 #'   data(gorillas, package = "inlabru")
 #'
@@ -73,7 +74,8 @@ gmap <- function(data, ...) {
 #' @family geomes for Raster data
 #'
 #' @examples
-#' if (require("ggplot2", quietly = TRUE)) {
+#' if (require("ggplot2", quietly = TRUE) &&
+#'   require(ggpolypath, quietly = TRUE)) {
 #'   # Load Gorilla data
 #'
 #'   data(gorillas, package = "inlabru")
@@ -110,7 +112,8 @@ gg <- function(data, ...) {
 #'
 #' @examples
 #' \dontrun{
-#' if (require("ggplot2", quietly = TRUE)) {
+#' if (require("ggplot2", quietly = TRUE) &&
+#'   require(ggpolypath, quietly = TRUE)) {
 #'   # Load the Gorilla data
 #'   data(gorillas, package = "inlabru")
 #'
@@ -494,7 +497,7 @@ gg.SpatialLines <- function(data, mapping = NULL, crs = NULL, ...) {
 #' @aliases gg.SpatialPolygons
 #' @name gg.SpatialPolygons
 #' @export
-#' @param data A `SpatialPolygons` object.
+#' @param data A `SpatialPolygons` or `SpatialPolygonsDataFrame` object.
 #' @param mapping Aesthetic mappings created by `aes` used to update the default
 #'                mapping. The default mapping is
 #'                `ggplot2::aes(x = long, y = lat, group = group)`.
@@ -502,7 +505,7 @@ gg.SpatialLines <- function(data, mapping = NULL, crs = NULL, ...) {
 #' @param ... Arguments passed on to `geom_polypath`.
 #' Unless specified by the user,
 #' the arguments `colour = "black"` (polygon colour) and
-#' `alpha = 0.1` (Alpha level for polygon filling).
+#' `alpha = 0.2` (Alpha level for polygon filling).
 #' @return A `ggpolypath::geom_polypath` object.
 #' @details Requires the `ggpolypath` package to ensure proper plotting, since
 #'   the `ggplot::geom_polygon` function doesn't always handle geometries with
@@ -518,7 +521,13 @@ gg.SpatialPolygons <- function(data, mapping = NULL, crs = NULL, ...) {
   if (!is.null(crs)) {
     data <- sp::spTransform(data, crs)
   }
+
   df <- ggplot2::fortify(data)
+  if (inherits(data, "SpatialPolygonsDataFrame")) {
+    data[["id"]] <- unique(df[["id"]])
+    df <- dplyr::left_join(df, as.data.frame(data), by = "id")
+  }
+
   cnames <- names(df)[1:2]
   if (requireNamespace("ggpolypath", quietly = TRUE)) {
     dmap <- ggplot2::aes(
@@ -539,17 +548,19 @@ gg.SpatialPolygons <- function(data, mapping = NULL, crs = NULL, ...) {
     dmap <- modifyList(dmap, mapping)
   }
 
-  params = list(...)
-  arg = list(data = df,
-             mapping = dmap)
+  params <- list(...)
+  arg <- list(
+    data = df,
+    mapping = dmap
+  )
 
   if (!any(names(params) %in% c("colour", "color"))) {
     arg$colour <- "black"
   }
   if (!any(names(params) %in% "alpha")) {
-    arg$alpha <- 0.1
+    arg$alpha <- 0.2
   }
-  arg = modifyList(arg, params)
+  arg <- modifyList(arg, params)
 
   if (requireNamespace("ggpolypath", quietly = TRUE)) {
     do.call(ggpolypath::geom_polypath, arg)
@@ -710,7 +721,7 @@ gg.inla.mesh <- function(data,
                          nx = 500, ny = 500,
                          ...) {
   requireNamespace("ggplot2")
-  if (is.null(color) && ("colour" %in% names(list(...)))) {
+  if (is.null(color) && ("colour" %in% ...names())) {
     color <- list(...)[["colour"]]
   }
   if (!is.null(color)) {
